#!/bin/bash
# Generate pdb of best docked pose ligand-receptor 

# setup python instruments

MGTDIR=~/mgltools    # change this according to the path of your mgltools dir 

pysh=$MGTDIR/bin/pythonsh
plig=$MGTDIR/MGLToolsPckgs/AutoDockTools/Utilities24/prepare_ligand4.py
prec=$MGTDIR/MGLToolsPckgs/AutoDockTools/Utilities24/prepare_receptor4.py
pgrid=$MGTDIR/MGLToolsPckgs/AutoDockTools/Utilities24/prepare_gpf4.py
ppar=$MGTDIR/MGLToolsPckgs/AutoDockTools/Utilities24/prepare_dpf4.py

n=50
while getopts ":l:r:x:y:z:n:" opt; do 
    case $opt in 
	l)
	    ligand=$OPTARG;
	    ;;
	r) 
	    receptor=$OPTARG; 
	    ;;
	x) 
	    x=$OPTARG; 
	    ;;
	y) 
	    y=$OPTARG; 
	    ;;
	z) 
	    z=$OPTARG; 
	    ;;
	n) 
	    n=$OPTARG; 
	    ;;
    esac
done

if [ $# == "0" ] ; then 
    echo ""
    echo "This script find best docking pose of a ligand to a receptor using Autodock4 "
    echo ""
    echo " Syntax: docking.bash  -l ligand -r receptor  [ opt ]"
    echo "          where  'ligand' assumes that file ligand.pdb exist"
    echo "                 'receptor' assumes that file receptor.pdb exist"
    echo "          "
    echo "          Options:"
    echo "          -x xcord -y ycord -z zcord "
    echo "          Specifies the ocking center on the receptor"
    echo "          "
    echo "          -n nrounds"
    echo "          Specifies the number of Docking rounds"
    exit
fi
echo $ligand

if [[ ! -v ligand  ]] ; then
    echo " Syntax: docking.bash  -l ligand -r receptor"
    echo " ligand not provided; provide filename of the ligand " 
    exit
fi
if [[ ! -v receptor ]] ; then
    echo " Syntax: docking.bash  -l ligand -r receptor"
    echo " receptor not provided; provide filename of the receptor " 
    exit
fi
if [ ! -f $ligand.pdb  ]; then
    echo "file" $ligand.pdb " not found"
    exit
fi
if [ ! -f $receptor.pdb  ]; then
    echo "file" $receptor.pdb " not found"
    exit
fi

echo " Generating ligand and receptor pdbqt file "

$pysh  $plig -l $ligand.pdb
$pysh  $prec -r $receptor.pdb

echo "Preparing the receptor gpf file"
$pysh $pgrid  -l $ligand.pdbqt  -r $receptor.pdbqt

# put the box center in the dpf file if this is given
if [ -v x -a -v y -a -v z ] ; then
    sed -i "s/gridcenter auto/gridcenter $x $y $z/g" $receptor.gpf  
fi

autogrid4 -p $receptor.gpf -l $receptor.glg

#Preparing the Docking parameter file 

$pysh  $ppar -l $ligand.pdbqt -r $receptor.pdbqt
sed -i "s/ga_run 10/ga_run $n/" ${ligand}_${receptor}.dpf

# All is done. We can run autodock 
autodock4 -p ${ligand}_${receptor}.dpf -l ${ligand}_${receptor}.dlg

best=`grep "^   1 |" ${ligand}_${receptor}.dlg | awk '{print $5}'`
free=`grep "^   1 |" ${ligand}_${receptor}.dlg  | awk '{print $3}'`

printf "run "$best" "$free "\n"

echo "generating  the best docked" $best " complex"

grep '^DOCKED' ${ligand}_${receptor}.dlg  | cut -c9- | awk -v run=$best '{if($1=="MODEL" && $2 == run) {ok=1}; if(ok==1 && $1 == "ATOM") {print}; if($1=="ENDMDL") {ok=0}}' | sed "s/  1   /-10   /g" > bestdocked.pdb

sed -i "s/ATOM  /HETATM/g" bestdocked.pdb

cat bestdocked.pdb $receptor.pdb  > complex.pdb 

exit 
